/* Atomix -- a little mind game about atoms and molecules.
 * Copyright (C) 1999 Jens Finke
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "atomix_object.h"

AtomixObject*
atomix_object_new(void)
{
    return atomix_object_new_args(ATOMIX_NONE, 0);
}

AtomixObject*
atomix_object_new_args(AtomixType type, guint id)
{
    AtomixObject* obj;

    obj = g_malloc(sizeof(AtomixObject));

    if(obj)
    {
	obj->type = type;
	obj->id = id;
    }
#ifdef DEBUG
    else
    {
	g_print("Not enough memory for AtomixObject allocation!\n");
    }
#endif

    return obj;
}

void
atomix_object_destroy(AtomixObject* obj)
{
    g_free(obj);
    obj = NULL;
}

AtomixObject*
atomix_object_copy(AtomixObject* obj)
{
    AtomixObject* obj_copy;
    obj_copy = atomix_object_new_args(obj->type, obj->id);
    g_assert(obj_copy!=NULL);

    return obj_copy;
}

void
atomix_object_print(AtomixObject* obj)
{
    gchar* type;
    switch(obj->type)
    {
    case ATOMIX_NONE:
	type = "NONE"; break;
    case ATOMIX_MOVEABLE:
	type = "MOVEABLE"; break;
    case ATOMIX_OBSTACLE:
	type = "OBSTACLE"; break;
    case ATOMIX_DECOR:
	type = "DECOR"; break;
    default:
	type = "OTHER"; break;
    }

    
    g_print("AtomixObject TYPE: %s   ID: %x\n",type, obj->id);   
}
