/* Atomix -- a little mind game about atoms and molecules.
 * Copyright (C) 1999 Jens Finke
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _ATOMIX_LEVEL_H 
#define _ATOMIX_LEVEL_H 

#include <gnome.h>
#include "global.h"
#include "playfield.h"
#include "theme.h"

typedef struct _Level       Level;

struct _Level
{
    gchar*       name;         /* name of the level */
    guint        time;         /* time to solve */
    gchar*       theme_name;   /* name of the used theme */
    PlayField*   goal;         /* determines the end of the level */
    PlayField*   playfield;    /* starting situation */
    gchar*       next;         /* name of the following level */
};


/**
 * Level file structure
 * 
 * level "<name>"
 * {
 *   time = <time in seconds>
 * [ theme = <theme name> ]
 *   next = "<level name>"
 *   
 *   playfield = { 
 *     { <type, id>, ... },
 *     { <type, id>, ... },
 *      ...
 *     } 
 *   
 *   goal = {
 *     { id, id, id, ... },
 *     ...
 *     }
 * }
 *
 * Grammar:
 * level_declaration ::= LEVEL identifier \{
 * 			 level_description
 * 			 \}
 * level_description ::=  time | theme | next | playfield | goal 
 * time ::= TIME = seconds
 * theme ::= THEME = identifier
 * next ::= NEXT = identifier;
 * playfield ::= PLAYFIELD = \{ {rows*}, \}
 * rows ::= \{ {cols*}, \},
 * cols ::= type, id,
 * goal ::= GOAL = \{ {goal_rows*}, \}
 * goal_rows ::= \{ {goal_cols*}, \},
 * goal_cols ::= id,
 * 
 */

typedef enum {
  LEVEL_TOKEN_INVALID = G_TOKEN_LAST,
  LEVEL_TOKEN_LEVEL,
  LEVEL_TOKEN_THEME,
  LEVEL_TOKEN_NEXT,
  LEVEL_TOKEN_PLAYFIELD,
  LEVEL_TOKEN_GOAL,
  LEVEL_TOKEN_TIME
} LevelTokenType;

typedef enum {
  LEVEL_NONE_READ, 
  LEVEL_LEVEL_READ,
  LEVEL_LEVEL_IDENTIFIER,
  LEVEL_OPEN_BRACE,
  LEVEL_CONTENT,
  LEVEL_CLOSED_BRACE,
  LEVEL_EQUAL_READ,
  LEVEL_VALUE_READ,
  LEVEL_READ_ROW,
  LEVEL_ROW_READ,
  LEVEL_COL_READ,
  LEVEL_TYPE_READ,
  LEVEL_COMMA_READ
} LevelFSM;

GHashTable* available_levels;

/* create/destroy levels */
Level*
level_new(void);

void
level_destroy(Level* l);

/* load & management functions */
Level*
level_load(gchar* path, LoadResult* result);

gboolean
level_parse_level(guint fd, Level* level);

Level*
level_load_xml(gchar* name);

void
level_save_xml(Level* level, gchar* filename);

void
level_create_hash_table(void);

void
level_destroy_hash_table(void);

gchar*
level_get_first_level(void);

#endif /* _ATOMIX_LEVEL_H */
